#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "convdecoder.h"

#undef SLOWACS
#define FASTACS
#undef NORM
#define MAXMETRIC 127

#define SW_SIZE		5	/* sliding window size=SW_SIZE*K*/

#define TWOTOTHEM 	256	/* 2^(K-1)*/
#define K 			(CONVCODER_NUMREGISTERS+1)
 
void deci2bin(unsigned char d, int size, unsigned char *b);
unsigned char bin2deci(unsigned char *b, int size);
unsigned char nxt_stat(unsigned char current_state, int in, char *memory_contents);
char soft_metric(char data, char guess);
 
char memory_contents[K];                   /* in + conv. encoder sr */
unsigned char in[TWOTOTHEM][TWOTOTHEM];          /* maps current/nxt sts to in */
unsigned char out[TWOTOTHEM][CONVCODER_MAXRATE];                 /* gives conv. encoder out */
unsigned char nextstate[TWOTOTHEM][CONVCODER_MAXRATE];              /* for current st, gives nxt given in */
char accum_err_metric[TWOTOTHEM][2];       /* accumulated error metrics */
char state_history[TWOTOTHEM][K * SW_SIZE + 1];  /* state history table */
char state_sequence[K * SW_SIZE + 1];            /* state sequence list */

unsigned char binary_out[CONVCODER_MAXRATE];                     /* vector to store binary enc out */
unsigned char branch_out[CONVCODER_MAXRATE];                     /* vector to store trial enc out */

unsigned char binary_state[K - 1];              /* binary value of current state */
unsigned char next_state_binary[K - 1];         /* binary value of next state */
    
 
void ConvDecoder(char *input, char *output, int len, int g[CONVCODER_MAXRATE][K], int rate) {
 
    int i, j, l, ll,k,t;                          /* loop variables */
   
    int m, n, number_of_states, depth_of_trellis, step, branch_metric,
        sh_ptr, sh_col, x, xx, h, hh, next_state, last_stop; /* misc variables */
 
/* ************************************************************************** */
 
    /* n is 2^1 = 2 for rate 1/2 */
    n = rate;
 
    /* m (memory length) = K - 1 */
    m = K - 1;

    /* number of states = 2^(K - 1) = 2^m for k = 1 */
    number_of_states = TWOTOTHEM;

    /* little degradation in performance achieved by limiting trellis depth
       to K * 5--interesting to experiment with smaller values and measure
       the resulting degradation. */
    depth_of_trellis = K * SW_SIZE;

    /* initialize data structures */
    for (i = 0; i < number_of_states; i++) {
        for (j = 0; j < number_of_states; j++)
            in[i][j] = 0;

        for (j = 0; j < n; j++) {
            nextstate[i][j] = 0;
            out[i][j] = 0;
        }

        for (j = 0; j <= depth_of_trellis; j++) {
            state_history[i][j] = 0;
        }

        /* initial accum_error_metric[x][0] = zero */
        accum_err_metric[i][0] = 0;
        /* by setting accum_error_metric[x][1] to MAXINT, we don't need a flag */
        /* so I don't get any more questions about this:  */
        /* MAXINT is simply the largest possible integer, defined in values.h */
        accum_err_metric[i][1] = MAXMETRIC;
    }

    for (j = 0; j < number_of_states; j++) {
        for (l = 0; l < n; l++) {
            next_state = nxt_stat(j, l, memory_contents);
            in[j][next_state] = l;

            /* now compute the convolutional encoder out given the current
               state number and the in value */
            for (k=0;k<n;k++)
	            branch_out[k] = 0;
    
            for (i = 0; i < K; i++) {
            	for (k=0;k<n;k++) 
	                branch_out[k] ^= memory_contents[i] & g[k][i];
	        }

            /* next state, given current state and in */
            nextstate[j][l] = next_state;
            /* out in decimal, given current state and in */
            out[j][l] = bin2deci(branch_out, 2);

        } /* end of l for loop */

    } /* end of j for loop */

    /* now we're going to rearrange the channel out so it has n rows,
       and n/2 columns where each row corresponds to a channel symbol for
       a given bit and each column corresponds to an encoded bit */
    len = len / n;
    

/* ************************************************************************** */
 
    /* End of setup. Start decoding of channel outs with forward
        traversal of trellis!  Stop just before encoder-flushing bits.  */
    for (t = 0; t < len - m; t++) {

       if (t <= m)
           /* assume starting with zeroes, so just compute paths from all-zeroes state */
           step = pow(2, m - t * 1);
        else
            step = 1;

        /* we're going to use the state history array as a circular buffer so
           we don't have to shift the whole thing left after each bit is
           processed so that means we need an appropriate pointer */
        /* set up the state history array pointer for this time t */
        sh_ptr = (int) ( ( t + 1 ) % (depth_of_trellis + 1) );

        /* repeat for each possible state */
        for (j = 0; j < number_of_states; j+= step) {
            /* repeat for each possible convolutional encoder out n-tuple */
            for (l = 0; l < n; l++) {
                branch_metric = 0;

                /* compute branch metric per channel symbol, and sum for all
                    channel symbols in the convolutional encoder out n-tuple */

                /* convert the decimal representation of the encoder out to binary */
                deci2bin(out[j][l], n, binary_out);

                /* compute branch metric per channel symbol, and sum for all
                    channel symbols in the convolutional encoder out n-tuple */
                for (ll = 0; ll < n; ll++) {
                    branch_metric = branch_metric + soft_metric(input[t+i], binary_out[ll]);

                } /* end of 'll' for loop */
                                

                /* now choose the surviving path--the one with the smaller accumlated
                    error metric... */
                if ( accum_err_metric[ nextstate[j][l] ] [1] > accum_err_metric[j][0] +
                    branch_metric ) {

                    /* save an accumulated metric value for the survivor state */
                    accum_err_metric[ nextstate[j][l] ] [1] = accum_err_metric[j][0] +
                        branch_metric;
 
                    /* update the state_history array with the state number of
                       the survivor */
                    state_history[ nextstate[j][l] ] [sh_ptr] = j;
 
                } /* end of if-statement */
            } /* end of 'l' for-loop */
        } /* end of 'j' for-loop -- we have now updated the trellis */
 
        /* for all rows of accum_err_metric, move col 2 to col 1 and flag col 2 */
        for (j = 0; j < number_of_states; j++) {
            accum_err_metric[j][0] = accum_err_metric[j][1];
            accum_err_metric[j][1] = MAXMETRIC;
        } /* end of 'j' for-loop */
 
 
        /* now start the traceback, if we've filled the trellis */
        if (t >= depth_of_trellis - 1) {
 
            /* initialize the state_sequence vector--probably unnecessary */
            for (j = 0; j <= depth_of_trellis; j++)
                state_sequence[j] = 0;
 
            /* find the element of state_history with the min. accum. error metric */
            /* since the outer states are reached by relatively-improbable runs
               of zeroes or ones, search from the top and bottom of the trellis in */
            x = MAXMETRIC;

            for (j = 0; j < ( number_of_states / 2 ); j++) {

                if ( accum_err_metric[j][0] < accum_err_metric[number_of_states - 1 - j][0] ) {
                    xx = accum_err_metric[j][0];
                    hh = j;
                }
                else {
                    xx = accum_err_metric[number_of_states - 1 - j][0];
                    hh = number_of_states - 1 - j;
                }
                if ( xx < x) {
                    x = xx;
                    h = hh;
                }

            } /* end 'j' for-loop */
            
            #ifdef NORM
                  if (x > MAXMETRIC) {
                for (j = 0; j < number_of_states; j++) {
                    accum_err_metric[j][0] = accum_err_metric[j][0] - x;
                    if (accum_err_metric[j][0] > MAXMETRIC)
                        accum_err_metric[j][0] = MAXMETRIC;
                } /* end 'j' for-loop */
            }
            #endif

  
            /* now pick the starting point for traceback */
            state_sequence[depth_of_trellis] = h;
 
            /* now work backwards from the end of the trellis to the oldest state
               in the trellis to determine the optimal path. The purpose of this
               is to determine the most likely state sequence at the encoder
                based on what channel symbols we received. */
            for (j = depth_of_trellis; j > 0; j--) {
                sh_col = j + ( sh_ptr - depth_of_trellis );
                if (sh_col < 0)
                   sh_col = sh_col + depth_of_trellis + 1;

                state_sequence[j - 1] = state_history[ state_sequence[j] ] [sh_col];
            } /* end of j for-loop */

            /* now figure out what in sequence corresponds to the state sequence
             in the optimal path */
            output[ t - depth_of_trellis + 1] =
                in[ state_sequence[0] ] [ state_sequence[1] ];

       } /* end of if-statement */
        
    } /* end of 't' for-loop */
 
    /* now decode the encoder flushing channel-out bits */
    for (t = len - m; t < len; t++) {

        /* set up the state history array pointer for this time t */
        sh_ptr = (int) ( ( t + 1 ) % (depth_of_trellis + 1) );

        /* don't need to consider states where in was a 1, so determine
         what is the highest possible state number where in was 0 */
        last_stop = number_of_states / pow(2, t - len + m);

        /* repeat for each possible state */
        for (j = 0; j < last_stop; j++) {
 
            branch_metric = 0;
            deci2bin(out[j][0], n, binary_out);
 
            /* compute metric per channel bit, and sum for all channel bits
                in the convolutional encoder out n-tuple */
            for (ll = 0; ll < n; ll++) {
                branch_metric = branch_metric + soft_metric( input[ll + t], binary_out[ll] );
            } /* end of 'll' for loop */

            /* now choose the surviving path--the one with the smaller total
                metric... */
            if ( (accum_err_metric[ nextstate[j][0] ][1] > accum_err_metric[j][0] +
                branch_metric) /*|| flag[ nextstate[j][0] ] == 0*/) {
 
                /* save a state metric value for the survivor state */
                accum_err_metric[ nextstate[j][0] ][1] = accum_err_metric[j][0] +
                    branch_metric;
 
                /* update the state_history array with the state number of
                    the survivor */
                state_history[ nextstate[j][0] ][sh_ptr] = j;
 
            } /* end of if-statement */
 
        } /* end of 'j' for-loop */
 
        /* for all rows of accum_err_metric, swap columns 1 and 2 */
        for (j = 0; j < number_of_states; j++) {
            accum_err_metric[j][0] = accum_err_metric[j][1];
            accum_err_metric[j][1] = MAXMETRIC;
        } /* end of 'j' for-loop */
 
        /* now start the traceback, if i >= depth_of_trellis - 1*/
        if (t >= depth_of_trellis - 1) {
 
            /* initialize the state_sequence vector */
            for (j = 0; j <= depth_of_trellis; j++) state_sequence[j] = 0;
 
            /* find the state_history element with the minimum accum. error metric */
            x = accum_err_metric[0][0];
            h = 0;
            for (j = 1; j < last_stop; j++) {
                if (accum_err_metric[j][0] < x) {
                    x = accum_err_metric[j][0];
                    h = j;
                } /* end if */
            } /* end 'j' for-loop */
            
            #ifdef NORM
            if (x > MAXMETRIC) {
                for (j = 0; j < number_of_states; j++) {
                    accum_err_metric[j][0] = accum_err_metric[j][0] - x;
                    if (accum_err_metric[j][0] > MAXMETRIC) {
                     accum_err_metric[j][0] = MAXMETRIC;
                    } /* end if */
                } /* end 'j' for-loop */
            }
            #endif

 
            state_sequence[depth_of_trellis] = h;
 
            /* now work backwards from the end of the trellis to the oldest state
                in the trellis to determine the optimal path. The purpose of this
                is to determine the most likely state sequence at the encoder
                based on what channel symbols we received. */
            for (j = depth_of_trellis; j > 0; j--) {

                sh_col = j + ( sh_ptr - depth_of_trellis );
                if (sh_col < 0)
                   sh_col = sh_col + depth_of_trellis + 1;

                state_sequence[j - 1] = state_history[ state_sequence[j] ][sh_col];
            } /* end of j for-loop */

            /* now figure out what in sequence corresponds to the
                optimal path */
 
             output[t - depth_of_trellis + 1] =
                in[ state_sequence[0] ][ state_sequence[1] ];

       } /* end of if-statement */
    } /* end of 't' for-loop */
 
    for (i = 1; i < depth_of_trellis - m; i++)
        output[len - depth_of_trellis + i] =
            in[ state_sequence[i] ] [ state_sequence[i + 1] ];

    return;
} /* end of function sdvd */

 

/* this metric is based on the algorithm given in Michelson and Levesque,
   page 323. */
char soft_metric(char data, char guess) {

    return(abs(data - (guess * 7)));
}


/* this function calculates the next state of the convolutional encoder, given
   the current state and the in data.  It also calculates the memory
   contents of the convolutional encoder. */
unsigned char nxt_stat(unsigned char current_state, int in, char *memory_contents) {
 
    int next_state;                       /* decimal value of next state */
    int i;                                /* loop variable */

    /* convert the decimal value of the current state number to binary */
    deci2bin(current_state, K - 1, binary_state);

    /* given the in and current state number, compute the next state number */
    next_state_binary[0] = in;
    for (i = 1; i < K - 1; i++)
        next_state_binary[i] = binary_state[i - 1];
 
    /* convert the binary value of the next state number to decimal */
    next_state = bin2deci(next_state_binary, K - 1);

    /* memory_contents are the ins to the modulo-two adders in the encoder */
    memory_contents[0] = in;
    for (i = 1; i < K; i++)
        memory_contents[i] = binary_state[i - 1];
 
    return(next_state);
}


/* this function converts a decimal number to a binary number, stored
   as a vector MSB first, having a specified number of bits with leading
   zeroes as necessary */
void deci2bin(unsigned char d, int size, unsigned char *b) {
    int i;
 
    for(i = 0; i < size; i++)
        b[i] = 0;
 
    b[size - 1] = d & 0x01;
 
    for (i = size - 2; i >= 0; i--) {
        d = d >> 1;
        b[i] = d & 0x01;
    }
}


/* this function converts a binary number having a specified
   number of bits to the corresponding decimal number
   with improvement contributed by Bryan Ewbank 2001.11.28 */
unsigned char bin2deci(unsigned char *b, int size) {
    int i, d;
 
    d = 0;
 
    for (i = 0; i < size; i++)
        d += b[i] << (size - i - 1);
 
    return(d);
}
