/** ALOE headers
 */
#include <phal_sw_api.h>
#include <swapi_utils.h>

#include "turbodecoder.h"

#include "typetools.h"

#include "itf_types.h"
#include "inputs.h"
#include "outputs.h"
#include "stats.h"


struct turbodecoderConf ccfg;

int longblock,numblocks;
int configured;

int iterations;

Tdec samples[INPUT_MAX_DATA];
char output_bits[8*INPUT_MAX_DATA];

#define USE_STATS

int process_input(int len) {
    int l,i;
    int rpm,wpm;
    char *input=input_data;
    char *output=output_bits;

    if (!configured)
        return;

    if (len!=typeSizeArray(TYPE_BITSTREAM,(RATE*longblock+TOTALTAIL)*numblocks)) {
        printf("TurboDecoder: Invalid input length: longcb %d numcb %d len %d\n",longblock,numblocks,len);
        return 0;
    }

   /* configure decoder options */
    ccfg.Long_CodeBlock=longblock;
    ccfg.Turbo_Dt=dynamic_halt;
    ccfg.Turbo_iteracions=8;
    /* for each codeblock */
    rpm=0;
    wpm=0;
    
    for (l=0;l<numblocks;l++) {
        for (i=0;i<RATE*longblock+TOTALTAIL;i++)
            samples[i]=(Tdec) (input[rpm+i]);
        iterations=TurboDecoder(samples,&output[wpm],&ccfg,&real_halt);
        rpm+=RATE*longblock+TOTALTAIL;
        wpm+=longblock;
    }

    char2bin(output,output_data,wpm);

    /* send throught output */
    SendItf(0,typeSizeArray(TYPE_BITSTREAM,wpm));

    return 1;
}

int process_control(int len) {
    configured=1;
    longblock=ctrlpkt.long_block;
    numblocks=ctrlpkt.num_blocks;
    xprintf("TurboDecoder: longcb=%d numcb=%d\n",longblock,numblocks);
}

/** Run function.
 * @return 1 if ok, 0 if error
 */
int RunCustom()
{
    return 1;
}

int InitCustom()
{
    configured=0;
    numblocks=0;
    longblock=0;
    return 1;
}

void ConfigInterfaces(){}




