
#define TYPE_BITSTREAM  0
#define TYPE_CHAR       1
#define TYPE_SHORT      2
#define TYPE_INT        3
#define TYPE_FLOAT      4


#define DIV32(a) (((a-1)>>5)+1)
#define DIV8(a) (((a-1)>>3)+1)
#define DIV4(a) (((a-1)>>2)+1)

/** bin2char
 * Converts a char-aligned bitstream to an array of bytes where each 
 * bit uses a byte
 */
void bin2char (char *input, char *output, int length);

/** bin2int8
 * Converts a char-aligned bitstream to an array of integers where each
 * bit uses an integer
 */
void bin2int8 (char *input, int *output, int length);

/** bin2float
 * Converts a char-aligned bitstream to an array of floats where each
 * bit uses an float
 */
void bin2float (char *input, float *output, int length);

/** char2bin
 * Converts an array of bytes where each bit uses an byte to a
 * char-algined bitstream
 */
void char2bin (char *input, char *output, int length);

/** int82bin
 * Converts an array of integers where each bit uses an integer to a
 * char-algined bitstream
 */
void int82bin (int *input, char *output, int length);

/** float2bin
 * Converts an array of floats where each bit uses a float to a
 * char-algined bitstream
 */
void float2bin (float *input, char *output, int length);

/** type2int
 * Converts an arbitrary type to an array of integers
 */
void type2int(void *input, int *output, int len, int datatype);

/** int2type
 * Converts an array of integers to an array of arbitrary type
 */
void int2type(int *input, void *output, int len, int datatype);

/** float2type
 * Converts an array of floats to an array of arbitrary type
 */
void float2type(float *input, void *output, int len, int datatype);

/** typeSizeArray
 * Returns the amount of bytes needed to store an array of nsamples
 * of arbitrary type
 */
int typeSizeArray(int datatype, int nsamples);

/** typeSizeArray
 * Returns the amount of bytes needed to store a sample
 * of arbitrary type
 */
int typeSize(int datatype);
