#include "typetools.h"


void bin2char (char *input, char *output, int length)
{
    char x;
    int i;

    for (i=0; i<length; i++)
    {
        if ((i%8)==0) {
            x=input[i/8];
        }
        output[i]=(x & 0x80)>>7;
        x<<=1;
    }
}

void bin2int8 (char *input, int *output, int length)
{
    char x;
    int i;

    for (i=0; i<length; i++)
    {
        if ((i%8)==0) {
            x=input[i/8];
        }
        output[i]=(x & 0x80)>>7;
        x<<=1;
    }
}


void bin2float (char *input, float *output, int length)
{
    char x;
    int i;

    for (i=0; i<length; i++)
    {
        if ((i%8)==0) {
            x=input[i/8];
        }
        output[i]=(x & 0x80)>>7;
        x<<=1;
    }
}

void char2bin (char *input, char *output, int length)
{
    int x;
    int i,c;

    x=0;
    c=0;
    for (i=0; i<length; i++)
    {
        if (input[i]) {
            x|=0x1;
        }
        c++;
        if (c==8 || i==length-1) {
            output[i/8]=((char) (x & 0xff))<<(8-c);
            x=0;
            c=0;
        } else {
            x<<=1;
        }
    }
}


void int82bin (int *input, char *output, int length)
{
    int x;
    int i,c;

    x=0;
    c=0;
    for (i=0; i<length; i++)
    {
        if (input[i]) {
            x|=0x1;
        }
        c++;
        if (c==8 || i==length-1) {
            output[i/8]=((char) (x & 0xff))<<(8-c);
            x=0;
            c=0;
        } else {
            x<<=1;
        }
    }
}



void float2bin (float *input, char *output, int length)
{
    int x;
    int i,c;

    x=0;
    c=0;
    for (i=0; i<length; i++)
    {
        if (input[i]) {
            x|=0x1;
        }
        c++;
        if (c==8 || i==length-1) {
            output[i/8]=((char) (x & 0xff))<<(8-c);
            x=0;
            c=0;
        } else {
            x<<=1;
        }
    }
}

void type2int(void *input, int *output, int len, int datatype)
{
    int i;
    char *inc;
    short *ins;
    int *ini;
    float *inf;

    switch(datatype)
    {
        case TYPE_BITSTREAM:
            bin2int8((char*) input, output, len);
            break;
        case TYPE_CHAR:
            inc=input;
            for (i=0;i<len;i++) {
                output[i] = (int) inc[i];
            }
            break;
        case TYPE_SHORT:
            ins=(short*) input;
            for (i=0;i<len;i++) {
                output[i] = (int) ins[i];
            }
            break;
        case TYPE_INT:
            ini=(int*) input;
            for (i=0;i<len;i++) {
                output[i] = (int) ini[i];
            }
            break;
        case TYPE_FLOAT:
            inf=(float*) input;
            for (i=0;i<len;i++) {
                output[i] = (int) inf[i];
            }
            break;
    }

}




void int2type(int *input, void *output, int len, int datatype)
{
    int i;
    char *inc;
    short *ins;
    int *ini;
    float *inf;

    switch(datatype)
    {
        case TYPE_BITSTREAM:
            int82bin(input, (char*) output, len);
            break;
        case TYPE_CHAR:
            inc=(char*) output;
            for (i=0;i<len;i++) {
                inc[i] = (char) input[i];
            }
            break;
        case TYPE_SHORT:
            ins=(short*) output;
            for (i=0;i<len;i++) {
                ins[i] = (int) input[i];
            }
            break;
        case TYPE_INT:
            ini=(int*) output;
            for (i=0;i<len;i++) {
                ini[i] = (int) input[i];
            }
            break;
        case TYPE_FLOAT:
            inf=(float*) output;
            for (i=0;i<len;i++) {
                inf[i] = (int) input[i];
            }
            break;
    }

}




void float2type(float *input, void *output, int len, int datatype)
{
    int i;
    char *inc;
    short *ins;
    int *ini;
    float *inf;

    switch(datatype)
    {
        case TYPE_BITSTREAM:
            printf("Not allowed\n");
            break;
        case TYPE_CHAR:
            inc=(char*) output;
            for (i=0;i<len;i++) {
                inc[i] = (char) input[i];
            }
            break;
        case TYPE_SHORT:
            ins=(short*) output;
            for (i=0;i<len;i++) {
                ins[i] = (int) input[i];
            }
            break;
        case TYPE_INT:
            ini=(int*) output;
            for (i=0;i<len;i++) {
                ini[i] = (int) input[i];
            }
            break;
        case TYPE_FLOAT:
            inf=(float*) output;
            for (i=0;i<len;i++) {
                inf[i] = (int) input[i];
            }
            break;
    }

}


int typeSizeArray(int datatype, int nsamples)
{
    switch(datatype)
    {
        case TYPE_BITSTREAM:
            return DIV8(nsamples);
        default:
            return nsamples*typeSize(datatype);
    }
}

int typeSize(int datatype)
{
    switch(datatype)
    {
        case TYPE_BITSTREAM:
            return sizeof(char);
        case TYPE_CHAR:
            return sizeof(char);
        case TYPE_SHORT:
            return sizeof(short);
        case TYPE_INT:
            return sizeof(int);
        case TYPE_FLOAT:
            return sizeof(float);
    }
}